package com.tzld.longarticle.recommend.server.service.recommend;

import cn.hutool.core.lang.UUID;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.CrawlerModeEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.ProducePlanInputSourceTypeEnum;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishPlanInputSourceTypesEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.PublishSingleVideoSourceStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticlePoolPromotionSourceStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.FeishuRobotIdEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.aigc.PublishContentMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.CrawlerContent;
import com.tzld.longarticle.recommend.server.model.dto.PublishContentDTO;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishAccount;
import com.tzld.longarticle.recommend.server.model.entity.crawler.PublishSortLog;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.*;
import com.tzld.longarticle.recommend.server.model.param.PublishContentParam;
import com.tzld.longarticle.recommend.server.model.vo.IdNameVO;
import com.tzld.longarticle.recommend.server.model.vo.WxContentDetailResponse;
import com.tzld.longarticle.recommend.server.model.vo.aigc.CommonListDataVO;
import com.tzld.longarticle.recommend.server.model.vo.aigc.ProduceContentListItemVO;
import com.tzld.longarticle.recommend.server.model.vo.aigc.ProducePlanDetailVO;
import com.tzld.longarticle.recommend.server.model.vo.aigc.ProducePlanInputSourceParam;
import com.tzld.longarticle.recommend.server.remote.WxFetchRemoteService;
import com.tzld.longarticle.recommend.server.remote.aigc.AIGCCrawlerPlanSaveService;
import com.tzld.longarticle.recommend.server.remote.aigc.AIGCProduceContentListService;
import com.tzld.longarticle.recommend.server.remote.aigc.AIGCProducePlanDetailService;
import com.tzld.longarticle.recommend.server.remote.aigc.AIGCProducePlanSaveService;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishAccountRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticlePoolPromotionSourceRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.DatastatSortStrategyRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.PublishSingleVideoSourceRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.VideoTitleReWriteRepository;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.math3.distribution.NormalDistribution;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.net.URLDecoder;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
@Slf4j
public class ArticlePromotionService {

    @Autowired
    LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    DatastatSortStrategyRepository datastatSortStrategyRepository;
    @Autowired
    AIGCCrawlerPlanSaveService aigcCrawlerPlanSaveService;
    @Autowired
    AIGCProducePlanDetailService aigcProducePlanDetailService;
    @Autowired
    AIGCProducePlanSaveService aigcProducePlanSaveService;
    @Autowired
    AIGCProduceContentListService aigcProduceContentListService;
    @Autowired
    ArticlePoolPromotionSourceRepository articlePoolPromotionSourceRepository;
    @Autowired
    WxFetchRemoteService wxFetchRemoteService;
    @Autowired
    PublishAccountRepository publishAccountRepository;
    @Autowired
    ArticleService articleService;
    @Autowired
    PublishSortLogRepository publishSortLogRepository;
    @Autowired
    PublishSingleVideoSourceRepository publishSingleVideoSourceRepository;
    @Autowired
    PublishContentMapper publishContentMapper;
    @Autowired
    AigcBaseMapper aigcBaseMapper;
    @Autowired
    VideoTitleReWriteRepository videoTitleReWriteRepository;
    @ApolloJsonValue("${articlePromotionProduceConfig:{}}")
    private Map<String, Map<String, Map<String, String>>> produceConfig;
    @Value("${topProducePlanId:}")
    private String topProducePlanId;
    @Value("${readOpenFissionRateThreshold:}")
    private Double readOpenFissionRateThreshold;

    private final List<String> contentPoolType = Arrays.asList("autoArticlePoolLevel1", "autoArticlePoolLevel3", "autoArticlePoolLevel4");

    /**
     *
     * @param readAvg：对照组分子
     * @param fansA：对照组分母
     * @param viewCount： 实验组分子
     * @param fansB: 实验组分母
     * @param confidence: 置信度
     * @return boolean: true / false
     */
    public static boolean isExperimentGroupStatisticallySuperior(double readAvg, int fansA, double viewCount, int fansB, double confidence) {

        if (fansA <= 0 || fansB <= 0) {
            throw new IllegalArgumentException("样本量必须大于零");
        }
        if (confidence <= 0 || confidence >= 1) {
            throw new IllegalArgumentException("置信水平必须在 (0,1) 区间");
        }
        double pA = readAvg / fansA;
        double pB = viewCount / fansB;

        if (pA <= 0) {
            throw new IllegalArgumentException("A组比例必须大于零");
        }
        double varA = pA * (1 - pA) / fansA;
        double varB = pB * (1 - pB) / fansB;
        double se = Math.sqrt(varA + varB);

        NormalDistribution normal = new NormalDistribution();
        double alpha = (1 - confidence) / 2;
        double zThreshold = normal.inverseCumulativeProbability(1 - alpha);
        double ciLow = (pB - pA - zThreshold * se) / pA;
        return ciLow > 0;
    }

    private Double nullSafe(Double value) {
        return value != null ? value : 0.0;
    }

    private Integer nullSafe(Integer value) {
        return value != null ? value : 0;
    }

    public List<DatastatSortStrategy> promotionWithReadOpenFissionRate(double threshold, List<DatastatSortStrategy> list) {
        if (CollectionUtils.isEmpty(list)) {
            throw new IllegalArgumentException("List cannot be empty");
        }

        return list.stream().filter(o -> {
            double denominator =
                    nullSafe(o.getFirstLevel()) +
                            nullSafe(o.getSecondFirstLevel()) +
                            nullSafe(o.getThirdFirstLevel());
            if (denominator == 0) return false;

            double numerator =
                    nullSafe(o.getFission0()) + nullSafe(o.getFission1()) + nullSafe(o.getFission2()) +
                            nullSafe(o.getSecondFission0()) + nullSafe(o.getSecondFission1()) + nullSafe(o.getSecondFission2()) +
                            nullSafe(o.getThirdFission0()) + nullSafe(o.getThirdFission1()) + nullSafe(o.getThirdFission2());

            double readRate = nullSafe(o.getReadRate());
            double firstReadRate = nullSafe(o.getFirstReadRate());
            // 计算 log10（阅读量)
            double logViewCount = (nullSafe(o.getViewCount()) > 0) ? Math.log10(nullSafe(o.getViewCount())) : 0;
            // 计算完整公式  score = 阅读均值倍数 * 阅读率 * T2裂变率 * log10（阅读量）
            double result = readRate * firstReadRate * (numerator / denominator) * logViewCount;
            return result > threshold;
        }).collect(Collectors.toList());
    }

    public void articlePromotion(String pos, String way, String accountNickName, String tag,
                                 Integer viewCountFilter, Double viewCountRateFilter, List<Integer> positionFilter) {
        String today = DateUtils.getCurrentDateStr("yyyyMMdd");
        String dateStrFilter = DateUtils.getBeforeDaysDateStr("yyyyMMdd", 10);
        // 获取内部表现
        List<DatastatSortStrategy> list;
        list = longArticleBaseMapper.getArticlePromotionCandidates(10000, dateStrFilter, positionFilter);
        // 使用阅读均值倍数+阅读量晋级
        List<DatastatSortStrategy> listStrategyV1 = list.stream()
                .filter(o -> o.getReadRate() >= viewCountRateFilter && o.getViewCount() >= viewCountFilter)
                .collect(Collectors.toList());
        List<String> StrategyV1WxSn = listStrategyV1.stream().map(DatastatSortStrategy::getWxSn).collect(Collectors.toList());
        List<DatastatSortStrategy> listStrategyV2;
        if (pos.equals("【2】")) {
            // 使用显著性检验晋级
            listStrategyV2 = list.stream()
                    .filter(o -> {
                        try {
                            return isExperimentGroupStatisticallySuperior(o.getAvgViewCount() * 1.1 * 30, o.getFans() * 30,
                                    o.getViewCount(), o.getFans(), 0.95);
                        }
                        catch (Exception e) {
                            log.error("显著性检验, 出现异常: {}", e.getMessage());
                            return false;
                        }
                    })
                    .collect(Collectors.toList());
        } else {
            // 使用新规则过滤
            listStrategyV2 = promotionWithReadOpenFissionRate(readOpenFissionRateThreshold, list);
        }
        listStrategyV2 = listStrategyV2.stream().filter(o -> !StrategyV1WxSn.contains(o.getWxSn())).collect(Collectors.toList());

        //策略 1 晋级
        filterAndAdd2CrawlerPlan(listStrategyV1, "策略V1", accountNickName, tag, pos, way, today);

        //策略 2 晋级
        filterAndAdd2CrawlerPlan(listStrategyV2, "策略V2", accountNickName, tag, pos, way, today);
    }

    private void filterAndAdd2CrawlerPlan(List<DatastatSortStrategy> list, String promotionStrategy, String accountNickName, String tag, String pos, String way, String today) {
        list = filterEarlyContent(list, true);
        log.info("优质{}文章数量: {}", accountNickName, list.size());
        List<DatastatSortStrategy> distinct = filterSameTitle(list);
        distinct.sort(Comparator.comparing(DatastatSortStrategy::getDateStr, Comparator.reverseOrder()));
        log.info("优质{}文章数量(去重后): {}", accountNickName, distinct.size());
        addUrlListToAccount(accountNickName, distinct, pos, way, today, tag, promotionStrategy);
    }

    private List<DatastatSortStrategy> filterEarlyContent(List<DatastatSortStrategy> list, Boolean filterVideoPool) {
        List<String> ghIds = list.stream().map(DatastatSortStrategy::getGhId).distinct().collect(Collectors.toList());
        List<PublishAccount> publishAccountList = publishAccountRepository.getAllByGhIdIn(ghIds);
        Map<String, Long> publishAccountCreateTimeMap = publishAccountList.stream().collect(Collectors.toMap(
                PublishAccount::getGhId, PublishAccount::getCreateTimestamp));
        list = list.stream().filter(o -> {
            long publishTime = DateUtils.dateStrToTimestamp(o.getDateStr(), "yyyyMMdd");
            Long accountCreateTime = publishAccountCreateTimeMap.get(o.getGhId());
            if (filterVideoPool) {
                return publishTime * 1000 > accountCreateTime
                        && !PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal().equals(o.getSourceType());
            } else {
                return publishTime * 1000 > accountCreateTime
                        && PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal().equals(o.getSourceType());
            }
        }).collect(Collectors.toList());
        return list;
    }

    private List<DatastatSortStrategy> filterSameTitle(List<DatastatSortStrategy> list) {
        List<DatastatSortStrategy> result = new ArrayList<>();
        List<String> titles = new ArrayList<>();
        for (DatastatSortStrategy datastatSortStrategy : list) {
            String title = datastatSortStrategy.getTitle();
            if (titles.contains(title)) {
                continue;
            }
            if (!TitleSimilarCheckUtil.isDuplicateContent(datastatSortStrategy.getTitle(), titles, TitleSimilarCheckUtil.ARTICLE_PROMOTION_THRESHOLD)) {
                result.add(datastatSortStrategy);
                titles.add(title);
            }
        }
        return result;
    }

    private void addUrlListToAccount(String accountNickName, List<DatastatSortStrategy> list, String pos, String way,
                                     String today, String tag, String promotionStrategy) {
        List<String> urlList = list.stream().map(DatastatSortStrategy::getLink).collect(Collectors.toList());
        if (!produceConfig.containsKey(accountNickName)) {
            log.info("account_nickname not in produceConfig: " + accountNickName);
            String planName = String.format("%d_%s_%s_%s【%s】_%s", list.size(), today, accountNickName, pos, way, today);
            aigcCrawlerPlanSaveService.createArticleUrlPlan(planName, urlList, tag, CrawlerModeEnum.ContentIDs.getVal());
            return;
        }
        List<String> publishContentIds = new ArrayList<>();
        List<String> filterUrlList = new ArrayList<>();
        try {
            String level = pos.equals("【1】") ? contentPoolType.get(0) : contentPoolType.get(1);
            String produceId = produceConfig.get(accountNickName).get(pos).get(way).trim();
            List<ProduceContentListItemVO> contentList = getProduceContentList(accountNickName, pos, way);
            // 获取已访问的标题和URL
            List<String> visitedTitleList = contentList.stream()
                    .flatMap(content -> Arrays.stream(new String[]{content.getReferContentTitle(), content.getTitle()}))
                    .distinct().collect(Collectors.toList());
            Set<String> visitedUrlIdList = contentList.stream().map(content -> getUrlId(content.getReferContentLink()))
                    .collect(Collectors.toSet());
            // 筛选URL和标题
            for (DatastatSortStrategy item : list) {
                String url = item.getLink();
                String urlId = getUrlId(item.getLink());
                String title = item.getTitle();
                String wxSn = item.getWxSn();
                if (visitedUrlIdList.contains(urlId)) {
                    continue;
                }
                if (TitleSimilarCheckUtil.isDuplicateContent(title, visitedTitleList, TitleSimilarCheckUtil.ARTICLE_PROMOTION_THRESHOLD)) {
                    continue;
                }
                // 能匹配到aigc发布记录则使用id抓取  否则使用url抓取
                String publishContentId = articleService.getPublishContentByWxSn(wxSn);
                if (StringUtils.hasText(publishContentId)) {
                    publishContentIds.add(publishContentId);
                    saveArticlePoolPromotionSource(Md5Util.encoderByMd5(publishContentId), wxSn, title, level);
                } else {
                    filterUrlList.add(url);
                    // 调用爬虫 detail 接口并保存数据
                    WxContentDetailResponse detail = getArticleDetail(url);
                    if (detail != null && StringUtils.hasText(detail.getChannelContentId())) {
                        saveArticlePoolPromotionSource(detail.getChannelContentId(), wxSn, title, level);
                    }
                }
            }
            if (CollectionUtils.isNotEmpty(filterUrlList)) {
                String planName = String.format("%d_%s_%s_%s【%s】_%s_%s", filterUrlList.size(), today, accountNickName, pos, way, today, promotionStrategy);
                log.info("url_len: " + list.size() + ", " + filterUrlList.size());
                IdNameVO<String> planInfo = aigcCrawlerPlanSaveService.createArticleUrlPlan(planName, filterUrlList, tag, CrawlerModeEnum.ContentIDs.getVal());
                if (StringUtils.hasText(produceId)) {
                    String inputSourceLabel = String.format("原始帖子-长文-微信公众号-内容添加计划-%s", planInfo.getName());
                    articleAddDependPlan(produceId, planInfo.getId(), inputSourceLabel, ProducePlanInputSourceTypeEnum.contentPlan.getVal());
                }
                log.info("{}, {}, produce plan not exist: {}, {}, {}", planInfo.getName(), planInfo.getId(), accountNickName, pos, way);
            }
            if (CollectionUtils.isNotEmpty(publishContentIds)) {
                String planName = String.format("%d_%s_%s_%s【%s】_%s_%s", publishContentIds.size(), today, accountNickName, pos, way, today, promotionStrategy);
                IdNameVO<String> planInfo = aigcCrawlerPlanSaveService.createArticleUrlPlan(planName, publishContentIds, tag, CrawlerModeEnum.PublishContentIds.getVal());
                if (StringUtils.hasText(produceId)) {
                    String inputSourceLabel = String.format("原始帖子-长文-微信公众号-内容添加计划-%s", planInfo.getName());
                    articleAddDependPlan(produceId, planInfo.getId(), inputSourceLabel, ProducePlanInputSourceTypeEnum.contentPlan.getVal());
                }
            }
            sendFeishuJobFinishMessage(accountNickName, filterUrlList.size(), publishContentIds.size(), promotionStrategy);
        } catch (Exception e) {
            log.error("articlePromotion error: ", e);
            FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.DAILY.getRobotId(),
                    "【文章晋升ERROR】\n" +
                            "晋级任务：" + accountNickName + "\n" +
                            "articlePromotion error: " + e.getMessage() + "\n"
                            + "<at user_id=\"all\">所有人</at> ");
        }
    }

    private void sendFeishuJobFinishMessage(String accountNickName, Integer urlListSize, Integer contentListSize, String promotionStrategy) {
        log.info("articlePromotion finish: 晋级任务:{}, id晋级数量：{}, url晋级数量：{}", accountNickName, contentListSize, urlListSize);
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.DAILY.getRobotId(),
                "【文章晋级job完成】\n" +
                        "晋级任务：" + accountNickName + "\n" +
                        "id晋级数量：" + contentListSize + "\n" +
                        "url晋级数量：" + urlListSize + "\n" +
                        "晋级策略: " + promotionStrategy + "\n");
    }

    private List<ProduceContentListItemVO> getProduceContentList(String accountNickName, String pos, String way) {
        List<String> planIdList = getProducePlanIdList(accountNickName, pos, way);
        CommonListDataVO<ProduceContentListItemVO> contentData = getProduceContentListByPlanIdList(planIdList);
        return contentData.getData();
    }

    public List<String> getProducePlanIdList(String accountNickname, String pos, String way) {
        List<String> res = new ArrayList<>();
        if (!produceConfig.containsKey(accountNickname)) {
            return res;
        }
        Map<String, Map<String, String>> accountConfig = produceConfig.get(accountNickname);
        for (String posKey : accountConfig.keySet()) {
            if (pos != null && !pos.equals(posKey)) {
                continue;
            }
            Map<String, String> posConfig = accountConfig.get(posKey);
            for (String wayKey : posConfig.keySet()) {
                if (way != null && !way.equals(wayKey)) {
                    continue;
                }
                String planId = posConfig.get(wayKey).trim();
                if (!planId.isEmpty()) {
                    res.add(planId);
                }
            }
        }
        return res;
    }

    public CommonListDataVO<ProduceContentListItemVO> getProduceContentListByPlanIdList(List<String> planIdList) {
        CommonListDataVO<ProduceContentListItemVO> result = new CommonListDataVO<>();
        if (planIdList.isEmpty()) {
            log.info("getProduceContentListByPlanIdList: planIdList empty");
            result.setData(new ArrayList<>());
            result.setTotalCount(0);
            return result;
        }
        int pageSize = 500;
        List<Integer> produceStatus = Arrays.asList(1, 2, 3, 4, 5, 6);
        CommonListDataVO<ProduceContentListItemVO> rawData = aigcProduceContentListService.list(planIdList, 1, 1, produceStatus);
        int totalCnt = rawData.getTotalCount();
        int pageNumMax = totalCnt / pageSize;
        List<ProduceContentListItemVO> allContent = new ArrayList<>();
        for (int i = 0; i <= pageNumMax; i++) {
            CommonListDataVO<ProduceContentListItemVO> pageData = aigcProduceContentListService.list(planIdList, i + 1, pageSize, produceStatus);
            allContent.addAll(pageData.getData());
        }
        List<ProduceContentListItemVO> filteredContent = new ArrayList<>();
        for (ProduceContentListItemVO content : allContent) {
            if (StringUtils.hasText(content.getTitle())) {
                filteredContent.add(content);
            }
        }
        result.setData(filteredContent);
        result.setTotalCount(filteredContent.size());
        return result;
    }

    public String getUrlId(String url) {
        if (url == null) {
            return null;
        }
        // 检查是否包含特定的前缀
        if (url.contains("https://mp.weixin.qq.com/s/")) {
            return url.split("https://mp.weixin.qq.com/s/")[1];
        }
        Map<String, String> params = new HashMap<>();
        try {
            String pureUrl = URLDecoder.decode(url, "utf-8");
            // 解析 URL 参数
            params = parseQueryString(pureUrl);

            String biz = params.get("http://mp.weixin.qq.com/s?__biz");
            String sn = params.get("sn");
            String mid = params.get("mid");
            String idx = params.get("idx");
            if (biz != null && sn != null && mid != null && idx != null) {
                return String.format("biz=%s_mid=%s_idx=%s_sn=%s", biz, mid, idx, sn);
            }
        } catch (Exception e) {
            log.error("get_url_id error: " + url);
        }

        return params.get("sn");
    }

    // 辅助方法：解析查询参数
    private Map<String, String> parseQueryString(String url) {
        Map<String, String> params = new java.util.HashMap<>();
        if (url.contains("?")) {
            String query = url.substring(url.indexOf("?") + 1);
            for (String param : query.split("&")) {
                String[] keyValue = param.split("=", 2);
                if (keyValue.length == 2) {
                    params.put(keyValue[0], keyValue[1]);
                }
            }
        }
        return params;
    }

    public WxContentDetailResponse getArticleDetail(String url) {
        if (url == null) {
            return null;
        }
        try {
            return wxFetchRemoteService.getContent(url);
        } catch (Exception e) {
            log.error("URL error: " + url);
        }
        return null;
    }

    private void saveArticlePoolPromotionSource(String channelContentId, String wxSn, String title, String level) {
        ArticlePoolPromotionSource articlePromotion = articlePoolPromotionSourceRepository.getByChannelContentId(channelContentId);
        if (Objects.isNull(articlePromotion)) {
            articlePromotion = new ArticlePoolPromotionSource();
            articlePromotion.setChannelContentId(channelContentId);
            articlePromotion.setTitle(title);
            articlePromotion.setTitleMd5(Md5Util.encoderByMd5(title));
            articlePromotion.setCreateTimestamp(System.currentTimeMillis());
            articlePromotion.setStatus(ArticlePoolPromotionSourceStatusEnum.WAITING.getCode());
            articlePromotion.setDeleted(StatusEnum.ZERO.getCode());
        }
        articlePromotion.setWxSn(wxSn);
        articlePromotion.setLevel(level);
        articlePromotion.setUpdateTimestamp(System.currentTimeMillis());
        articlePoolPromotionSourceRepository.save(articlePromotion);
    }

    private void articleAddDependPlan(String produceId, String inputSourceValue, String inputSourceLabel, Integer inputSourceType) {
        try {
            // 获取生产计划的详细信息
            ProducePlanDetailVO detail = aigcProducePlanDetailService.articleGetProducePlanDetail(produceId);
            if (detail == null) {
                FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.JOB.getRobotId(),
                        "文章晋升ERROR:\n" +
                                "articleAddDependPlan getProducePlan error: planId:" + produceId);
                return;
            }
            // 获取依赖计划 ID 列表
            List<ProducePlanInputSourceParam> inputSources = detail.getInputSourceGroups().get(0).getInputSources();
            List<String> dependValues = new ArrayList<>();
            for (ProducePlanInputSourceParam inputSource : inputSources) {
                dependValues.add(inputSource.getInputSourceValue());
            }
            // 如果计划 ID 已存在，直接返回
            if (dependValues.contains(inputSourceValue)) {
                log.info("depend_value exist: {}", inputSourceValue);
                return;
            }
            aigcProducePlanSaveService.save(inputSourceLabel, inputSourceValue, detail, inputSourceType);
        } catch (Exception e) {
            FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.JOB.getRobotId(),
                    "文章晋升ERROR:\n" +
                            "articleAddDependPlan error: " + e.getMessage());
        }
    }

    public void articleVideoPoolPromotion(Integer viewCountFilter, Double viewCountRateFilter,
                                          List<Integer> positionFilter, Integer poolLevel) {
        String dateStrFilter = DateUtils.getBeforeDaysDateStr("yyyyMMdd", 10);
        // 获取内部表现
        List<DatastatSortStrategy> list = longArticleBaseMapper.getArticlePromotion(viewCountFilter, viewCountRateFilter,
                10000, dateStrFilter, positionFilter);
        list = filterEarlyContent(list, false);
        log.info("视频内容池优质文章数量: {}", list.size());
        List<DatastatSortStrategy> distinct = filterSameTitle(list);
        distinct.sort(Comparator.comparing(DatastatSortStrategy::getDateStr, Comparator.reverseOrder()));
        log.info("视频内容池优质文章数量(去重后): {}", distinct.size());
        saveArticleVideoPoolPromotion(distinct, poolLevel);
    }

    private void saveArticleVideoPoolPromotion(List<DatastatSortStrategy> promotionList, Integer poolLevel) {
        if (CollectionUtils.isEmpty(promotionList)) {
            return;
        }
        List<String> ghIds = promotionList.stream().map(DatastatSortStrategy::getGhId).distinct().collect(Collectors.toList());
        List<String> dateStr = promotionList.stream().map(DatastatSortStrategy::getDateStr).distinct().collect(Collectors.toList());
        List<PublishSortLog> sortLogList = publishSortLogRepository.getByGhIdInAndDateStrIn(ghIds, dateStr);
        Map<String, Map<String, List<PublishSortLog>>> sortLogMap = sortLogList.stream()
                .collect(Collectors.groupingBy(PublishSortLog::getGhId,
                        Collectors.groupingBy(PublishSortLog::getDateStr)));
        int promotionNum = 0;
        for (DatastatSortStrategy data : promotionList) {
            List<PublishSortLog> logs = sortLogMap.get(data.getGhId()).get(data.getDateStr());
            if (CollectionUtils.isEmpty(logs)) {
                continue;
            }
            String sourceId = null;
            for (PublishSortLog publishSortLog : logs) {
                if (data.getTitle().equals(publishSortLog.getTitle())) {
                    sourceId = publishSortLog.getSourceId();
                    break;
                }
            }
            if (!StringUtils.hasText(sourceId)) {
                continue;
            }
            PublishSingleVideoSource singleVideoSource = publishSingleVideoSourceRepository.getByContentTraceId(sourceId);
            if (Objects.isNull(singleVideoSource)) {
                continue;
            }
            PublishSingleVideoSource exists = publishSingleVideoSourceRepository.getByUpLevelSource(sourceId);
            if (Objects.nonNull(exists)) {
                continue;
            }
            PublishSingleVideoSource upLevel = new PublishSingleVideoSource();
            BeanUtils.copyProperties(singleVideoSource, upLevel);
            singleVideoSource.setBadStatus(1);
            singleVideoSource.setUpLevelTimestamp(System.currentTimeMillis() / 1000);
            publishSingleVideoSourceRepository.save(singleVideoSource);
            upLevel.setId(null);
            upLevel.setUpLevelSource(singleVideoSource.getContentTraceId());
            upLevel.setContentTraceId("video" + UUID.randomUUID().toString().replace("-", ""));
            upLevel.setFlowPoolLevel(poolLevel);
            upLevel.setStatus(PublishSingleVideoSourceStatusEnum.DEFAULT.getCode());
            upLevel = publishSingleVideoSourceRepository.save(upLevel);

            List<VideoTitleReWrite> videoTitleReWrites = videoTitleReWriteRepository.getByContentTraceId(singleVideoSource.getContentTraceId());
            if (CollectionUtils.isNotEmpty(videoTitleReWrites)) {
                for (VideoTitleReWrite videoTitleReWrite : videoTitleReWrites) {
                    videoTitleReWrite.setId(null);
                    videoTitleReWrite.setContentTraceId(upLevel.getContentTraceId());
                    videoTitleReWriteRepository.save(videoTitleReWrite);
                }
            }

            VideoEndScreenTransformationTask task = new VideoEndScreenTransformationTask();
            task.setVideoOriginId(upLevel.getId());
            task.setTitle(upLevel.getArticleTitle());
            task.setOssPath(upLevel.getVideoOssPath());
            longArticleBaseMapper.saveVideoEndScreenTransformationTask(task);

            promotionNum++;
        }
        ContentPoolEnum poolEnum = ContentPoolEnum.from(poolLevel);
        sendFeishuJobFinishMessage(poolEnum.getDescription(), promotionNum);
    }

    private void sendFeishuJobFinishMessage(String accountNickName, Integer promotionNum) {
        log.info("articleVideoPoolPromotion finish: 晋级任务:{}, 晋级数量：{}", accountNickName, promotionNum);
        FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.DAILY.getRobotId(),
                "【视频晋级任务完成】\n" +
                        "晋级任务：" + accountNickName + "\n" +
                        "晋级数量：" + promotionNum + "\n");
    }

    @XxlJob("topContentReSendJob")
    public ReturnT<String> topContentReSend(String param) {
        String dateStr = DateUtils.getBeforeDaysDateStr("yyyyMMdd", 60);
        if (StringUtils.hasText(param)) {
            dateStr = param;
        }
        List<DatastatSortStrategy> topList = longArticleBaseMapper.getTopContent(dateStr);
        List<String> ghIds = topList.stream().map(DatastatSortStrategy::getGhId).distinct().collect(Collectors.toList());
        List<PublishAccount> publishAccountList = publishAccountRepository.getAllByGhIdIn(ghIds);
        Map<String, String> publishContentMap = publishAccountList.stream().collect(Collectors.toMap(
                PublishAccount::getGhId, PublishAccount::getId));
        List<PublishContentParam> publishContentParamList = topList.stream().map(top -> {
            PublishContentParam item = new PublishContentParam();
            item.setTitle(top.getTitle());
            String accountId = publishContentMap.get(top.getGhId());
            item.setPublishAccountId(accountId);
            return item;
        }).collect(Collectors.toList());
        List<PublishContentDTO> publishContentList = publishContentMapper.getPublishContentByTitle(publishContentParamList);
        List<String> existsTitles = aigcBaseMapper.getProduceContentTitleByPlanId(topProducePlanId);
        List<String> channelContentIds = publishContentList.stream().map(PublishContentDTO::getCrawlerChannelContentId).collect(Collectors.toList());
        List<CrawlerContent> crawlerContentList = aigcBaseMapper.getCrawlerContentByChannelContentIdIn(channelContentIds);
        Map<String, Long> crawlerContentMap = crawlerContentList.stream().collect(Collectors.toMap(
                CrawlerContent::getChannelContentId, CrawlerContent::getId));
        for (PublishContentDTO item : publishContentList) {
            if (TitleSimilarCheckUtil.isDuplicateContent(item.getTitle(), existsTitles, TitleSimilarCheckUtil.ARTICLE_PROMOTION_THRESHOLD)) {
                continue;
            }
            existsTitles.add(item.getTitle());
            String inputSourceLabel = String.format("原始帖子-长文-微信公众号-内容IDs-%s", item.getTitle());
            Long crawlerContentId = crawlerContentMap.get(item.getCrawlerChannelContentId());
            if (Objects.isNull(crawlerContentId)) {
                continue;
            }
            articleAddDependPlan(topProducePlanId, String.valueOf(crawlerContentId), inputSourceLabel,
                    ProducePlanInputSourceTypeEnum.contentID.getVal());
        }
        return ReturnT.SUCCESS;
    }
}
