package com.tzld.longarticle.recommend.server.service.recommend.score.strategy;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.enums.StatusEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.AccountCategory;
import com.tzld.longarticle.recommend.server.repository.aigc.CrawlerMetaArticleRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.AccountCategoryRepository;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.score.AccountCategoryWeightConfig;
import com.tzld.longarticle.recommend.server.service.recommend.score.Score;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.*;

@Component
@Slf4j
public class CategoryStrategy implements ScoreStrategy {

    @Autowired
    CrawlerMetaArticleRepository crawlerMetaArticleRepository;
    @Autowired
    AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    AccountCategoryRepository accountCategoryRepository;

    @ApolloJsonValue("${accountCategoryWeightConfig:{}}")
    private Map<String, AccountCategoryWeightConfig[]> accountCategoryWeightConfigMap;

    @Value("${category.active.version:1}")
    private Integer activeVersion;

    @ApolloJsonValue("${category.strategy.add.account.list:[]}")
    private List<String> categoryAddAccountList;

    @ApolloJsonValue("${category.strategy.add.score:{}}")
    private Map<String, Double> categoryAddScoreConfig;

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();
        List<Score> scores = new ArrayList<>();
        if (CollectionUtils.isEmpty(param.getContents())) {
            return scores;
        }
        List<AccountCategory> accountCategoryList = accountCategoryRepository.getByGhIdAndStatusAndVersion(
                param.getGhId(), StatusEnum.ONE.getCode(), activeVersion);
        AccountCategory accountCategory = null;
        if (CollectionUtils.isNotEmpty(accountCategoryList)) {
            accountCategory = accountCategoryList.stream()
                    .sorted(Comparator.comparing(AccountCategory::getDt, Comparator.reverseOrder()))
                    .findFirst().get();
        }
        for (Content content : param.getContents()) {
            if (CollectionUtils.isEmpty(content.getCategory())) {
                continue;
            }
            Score score = new Score();
            score.setStrategy(this);
            score.setContentId(content.getId());
            double scoreValue = 0.0;
            double categoryWeight = 0.0;
            for (String category : content.getCategory()) {
                if (Objects.nonNull(accountCategory) && StringUtils.hasText(accountCategory.getCategoryMap())) {
                    JSONObject categoryWeightMap = JSONObject.parseObject(accountCategory.getCategoryMap());
                    categoryWeight = categoryWeightMap.getDoubleValue(category);
                    scoreValue += categoryWeight;
                }
                if (categoryAddAccountList.contains(param.getAccountName()) && categoryWeight >= 0) {
                    Double addScore = categoryAddScoreConfig.get(category);
                    if (Objects.nonNull(addScore)) {
                        scoreValue += addScore;
                    }
                }
            }
            score.setScore(scoreValue);
            scores.add(score);
        }
        log.info("CategoryStrategy cost:{}", System.currentTimeMillis() - start);
        return scores;
    }

}
